package co.fast.flutter.sdk

import android.content.Intent
import android.util.Log
import androidx.annotation.NonNull
import co.fast.android.sdk.Fast
import co.fast.android.sdk.checkout.FastCheckoutEvent
import co.fast.android.sdk.checkout.FastCheckoutWithCartIdOptions
import co.fast.android.sdk.checkout.FastCheckoutWithProductsOptions
import co.fast.android.sdk.checkout.internal.FastCheckoutActivity
import co.fast.android.sdk.checkout.internal.FastCheckoutActivityResult
import co.fast.android.sdk.checkout.internal.FastCheckoutActivityResultContract
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.common.StandardMethodCodec
import java.util.Date

/** FastSdkPlugin */
class FastSdkPlugin : FlutterPlugin, ActivityAware, FastFlutter.FastFlutterApi {
    private var flutterPluginBinding: FlutterPlugin.FlutterPluginBinding? = null
    private var appId: String = ""

    private var activityPluginBinding: ActivityPluginBinding? = null

    private var fastEventChannel: EventChannel? = null
    private var fastEventStreamHandler: FastCheckoutEventChannelStreamHandler? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        this.flutterPluginBinding = flutterPluginBinding

        fastEventStreamHandler = FastCheckoutEventChannelStreamHandler()
        fastEventChannel = EventChannel(
            flutterPluginBinding.binaryMessenger,
            CHANNEL_NAME_FAST_EVENTS,
            StandardMethodCodec(FastFlutter.FastFlutterApi.getCodec() as StandardMessageCodec)
        )
        fastEventChannel?.setStreamHandler(fastEventStreamHandler)

        FastFlutter.FastFlutterApi.setup(flutterPluginBinding.binaryMessenger, this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        this.flutterPluginBinding = binding
        fastEventChannel = null
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityPluginBinding = binding
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activityPluginBinding = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activityPluginBinding = binding
    }

    override fun onDetachedFromActivity() {
        activityPluginBinding = null
        fastEventChannel = null
    }

    override fun configure(appId: String, result: FastFlutter.Result<Void>?) {
        Log.i(TAG, "configure() called with appId: $appId")
        this.appId = appId
        flutterPluginBinding?.let { binding ->
            Fast.init(binding.applicationContext) { options ->
                options.appId = appId
            }
            result?.success(null)
        }
    }

    override fun reset(result: FastFlutter.Result<Void>?) {
        Log.i(TAG, "reset() called")
        Fast.reset()
        result?.success(null)
    }

    override fun checkoutWithProducts(
        products: MutableList<FastFlutter.Product>,
        currencyCode: String?,
        couponCode: String?,
        result: FastFlutter.Result<Void>?
    ) {
        Log.i(
            TAG,
            "checkoutWithProducts() called. Products: $products, currencyCode: $currencyCode, couponCode: $couponCode"
        )

        val context = flutterPluginBinding!!.applicationContext
        val intent = FastCheckoutActivity.createIntent(
            context,
            FastCheckoutActivityResultContract.Args.createForItems(
                appId,
                FastCheckoutWithProductsOptions().apply {
                    addProducts(products.toProducts())
                    this.currencyCode = currencyCode
                    this.couponCode = couponCode
                }
            )
        )

        listenOnceForOnActivityResult(result)
        activityPluginBinding?.activity?.startActivityForResult(intent, REQUEST_CODE)
    }

    override fun checkoutWithCartId(
        cartId: String,
        currencyCode: String?,
        result: FastFlutter.Result<Void>?
    ) {
        Log.i(TAG, "checkoutWithCartId() called. cartId: $cartId, currencyCode: $currencyCode")

        val context = flutterPluginBinding!!.applicationContext
        val intent = FastCheckoutActivity.createIntent(
            context,
            FastCheckoutActivityResultContract.Args.createForCartId(
                appId,
                FastCheckoutWithCartIdOptions().apply {
                    this.cartId = cartId
                    this.currencyCode = currencyCode
                }
            )
        )

        listenOnceForOnActivityResult(result)
        activityPluginBinding?.activity?.startActivityForResult(intent, REQUEST_CODE)
    }

    override fun _unused(event: FastFlutter.FastEvent, result: FastFlutter.Result<Void>?) {
        TODO("Not yet implemented")
    }

    /**
     * Adds an listener for one occurrence of onActivityResult, then unregisters itself afterwords.
     */
    private fun listenOnceForOnActivityResult(methodCallResult: FastFlutter.Result<Void>?) {
        activityPluginBinding?.addActivityResultListener(object :
            PluginRegistry.ActivityResultListener {
            override fun onActivityResult(
                requestCode: Int,
                resultCode: Int,
                data: Intent?
            ): Boolean {
                Log.d(
                    TAG,
                    "listenOnceForOnActivityResult(): onAR: $requestCode, $resultCode, $data"
                )
                if (requestCode != REQUEST_CODE) {
                    return true
                }

                // Emit FastEvents if any were returned
                if (data != null) {
                    val result: FastCheckoutActivityResult = requireNotNull(
                        data.getParcelableExtra(FastCheckoutActivity.INTENT_RESPONSE_DATA)
                    )

                    when (result.events.isEmpty()) {
                        true -> emitFastEvent(FastCheckoutEvent.SDKClosed(null))
                        else -> result.events.forEach { emitFastEvent(it) }
                    }
                } else {
                    // TODO Remove this fake event STOPSHIP
                    emitFastEvent(
                        FastCheckoutEvent.OrderCreated(
                            FastCheckoutEvent.PurchaseInfo(
                                platformId = "platformId",
                                revenue = "9.99",
                                tax = "0.99",
                                currencyCode = "USD",
                                coupon = null,
                                checkoutTime = Date(),
                                shipping = "0.00",
                                items = listOf(),
                            )
                        )
                    )

                    emitFastEvent(FastCheckoutEvent.SDKClosed(null))
                }

                activityPluginBinding?.removeActivityResultListener(this)
                methodCallResult?.success(null)
                return true
            }
        })
    }

    private fun emitFastEvent(event: FastCheckoutEvent) {
        Log.d(TAG, "emitFastEvent: $event")
        checkNotNull(fastEventStreamHandler).emit(event)
    }

    companion object {
        private val TAG = FastSdkPlugin::class.java.simpleName
        private const val REQUEST_CODE = 38782
        private const val CHANNEL_NAME_FAST_EVENTS = "co.fast.flutter.sdk/fast_events"
    }
}
